package vio.app;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import vio.tracker.*;
import vio.util.*;
import modin.util.*;

public class trackerPano extends Frame {

	public trackerPano() {

		setTitle(" trackerPano ");
		menuBar = new MenuBar();
		setMenuBar(menuBar);

		fileMenu = new Menu();
		fileMenu.setLabel("File");
		menuBar.add(fileMenu);

		fileMenuOpen = new MenuItem();
		fileMenuOpen.setLabel("open panorama");
		fileMenu.add(fileMenuOpen);

		fileMenu.addSeparator();

		fileMenuQuit = new MenuItem();
		fileMenuQuit.setLabel("quit");
		fileMenu.add(fileMenuQuit);

		helpMenu = new Menu();
		helpMenu.setLabel("Help");
		menuBar.add(helpMenu);

		helpMenuHelp = new MenuItem();
		helpMenuHelp.setLabel("quick help");
		helpMenu.add(helpMenuHelp);

		helpMenuInfo = new MenuItem();
        	helpMenuInfo.setLabel("tracker info");
		helpMenu.add(helpMenuInfo);

		helpMenu.addSeparator();

        	helpMenuAbout = new MenuItem();
        	helpMenuAbout.setLabel("about");
		helpMenu.add(helpMenuAbout);

		addWindowListener (
			new WindowAdapter () {
				public void windowClosing (WindowEvent evt) {
					System.exit(0);
				}
			}
		);

		fileMenuOpen.addActionListener (
			new ActionListener () {
				public void actionPerformed (ActionEvent evt) {
					fileMenuOpenActionPerformed (evt);
				}
			}
		);

		fileMenuQuit.addActionListener (
			new ActionListener () {
				public void actionPerformed (ActionEvent evt) {
					System.exit(0);
				}
			}
		);

		helpMenuHelp.addActionListener (
			new ActionListener () {
        			public void actionPerformed (ActionEvent evt) {
					qhlp.show();
				}
			}
        	);

        	helpMenuInfo.addActionListener (
			new ActionListener () {
        			public void actionPerformed (ActionEvent evt) {
					showTrackerInfo();
				}
        		}
        	);

         	helpMenuAbout.addActionListener (
			new ActionListener () {
        			public void actionPerformed (ActionEvent evt) {
					abtDlg.show();
				}
        		}
        	);

		messageLabel = new Label("                                            ");
		messageLabel.setFont(new Font("Dialog", Font.PLAIN, 12));
		messageLabel.setBounds(10,45,230,20);
		add(messageLabel);

		abtDlg = new aboutDialog(this,"about...", true);
		abtDlg.setHeadline("trackerPano "+version);

		qhlp = new quickHelp(this);
		qhlp.setHeadline("trackerPano");
		qhlp.appendLine("This little application allows you to watch panorama images controlled by the i-glasses' head tracker.\n\n");
		qhlp.appendLine("To watch a panorama simply open the image via the file menu. A fullscreen image will pop up immediately. ");
		qhlp.appendLine("A click on the screen closes the panorama, and the menubar is visible again.\n\n");
		qhlp.appendLine("Image formats that are currently supported are GIF and JPG files. Make sure that your images have the right extensions, others will be ignored.\n\n");
		qhlp.appendLine("If this program doesn't work, you probably have not a proper Java installation. You will need at least the following components:\n\n");
		qhlp.appendLine("- JDK 1.1 or later\n");
		qhlp.appendLine("- Java Communication API 2.0 or later\n");
		qhlp.appendLine("You will find the latest version of the driver, this program and links to the packages mentioned above at the following webpage:\n");
		qhlp.appendLine("http://iglasses.weirdoz.org/\n\n");
		qhlp.appendLine("trackerPano was written by\nMartin Kaltenbrunner, modin@yuri.at\n");
		qhlp.appendLine("and is available under the GNU Public License, which can be read in the file license.html");

		pack();
	}

	private void fileMenuOpenActionPerformed (ActionEvent evt) {
		killAll();

		fd = new FileDialog(this,"open panorama",FileDialog.LOAD);
		if (lastPath!=null) fd.setDirectory(lastPath);
		fd.show();
 
		String path = fd.getDirectory();
		String imagePath = path+fd.getFile();

		String imageType="";
		StringTokenizer st = new StringTokenizer(imagePath,".");
	
		try { st.nextElement(); imageType=(String)st.nextElement(); }
		catch (Exception e) {}
	
		if ( (imageType.equalsIgnoreCase("gif")) || (imageType.equalsIgnoreCase("jpg")) || (imageType.equalsIgnoreCase("jpeg")) )
		{
			setCursor(new Cursor(Cursor.WAIT_CURSOR));
			Image svrImage = loadImage(imagePath);
			initTracker();
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			lastPath = path;
			showPano(svrImage);
		}
	}

	private Image loadImage(String path) {

		messageLabel.setText("loading image...");

		MediaTracker mt = new MediaTracker(this);
                Image img = Toolkit.getDefaultToolkit().getImage(path);

		mt.addImage(img, 0);
		try {mt.waitForAll(); }
		catch (InterruptedException e){} ;

		messageLabel.setText("");
		return img;
	}
  
	private void killAll() {

		try { pCanvas.stop(); pCanvas=null; }
		catch (Exception e) {}

		try { fsw.remove(pCanvas); fsw.Hide(); fsw=null; }
		catch (Exception e) {}

		try { tracker.close(); tracker=null; }
		catch (Exception e) {}	
	}

	private void showTrackerInfo() {

		if (myTrackerInfo==null) {
			setCursor(new Cursor(Cursor.WAIT_CURSOR));
			initTracker();
			myTrackerInfo = tracker.getTrackerInfo();
			infDlg = new trackerInfoDialog(this,myTrackerInfo);
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}

		infDlg.show();
	}
  
	private void initTracker() {

		messageLabel.setText("initializing tracker...");

		if (trackerPortName==null) {
			tracker = new TrackerManager(false);
			boolean trackerAvailable = tracker.init();

			while (!trackerAvailable) {
				tracker = new TrackerManager(false);
				trackerAvailable=tracker.init();

				if (!trackerAvailable)
				messageLabel.setText("please check the tracker ...");

				/*try { Thread.sleep(100); }
				catch (InterruptedException e) {}*/
			}

			trackerPortName = tracker.getTrackerPortName();

		} else {
			tracker = new TrackerManager(trackerPortName,false);
			boolean trackerAvailable=tracker.init();

			while (!trackerAvailable) {
				tracker = new TrackerManager(trackerPortName,false);
				trackerAvailable=tracker.init();

				if (!trackerAvailable)
				messageLabel.setText("please check the tracker ...");

				/*try { Thread.sleep(100); }
				catch (InterruptedException e) {}*/
			}
		}	

		messageLabel.setText("");
	}

	private void showPano (Image img) {

		fsw = new fullScreenWindow(this);
		pCanvas = new panoCanvas(img, fsw, tracker);

	   	fsw.Show(640,480);
	   	pCanvas.start();
	}

	public static void main(java.lang.String[] args) {
		new trackerPano().show();
	}
  
	private MenuBar menuBar;
	private Menu fileMenu;
	private Menu helpMenu;
	private MenuItem fileMenuOpen;
	private MenuItem fileMenuQuit;
	private MenuItem helpMenuHelp;
	private MenuItem helpMenuInfo;
	private MenuItem helpMenuAbout;

	private aboutDialog abtDlg;
	private trackerInfoDialog infDlg;
	private quickHelp qhlp;
  
	private Label messageLabel;
	private FileDialog fd;
  
	private fullScreenWindow fsw;
	private TrackerManager tracker;
	private TrackerInfo myTrackerInfo;
	private panoCanvas pCanvas;
	private String trackerPortName;

	private String version = TrackerManager.getVersion();
	private String lastPath;
}

