package vio.tracker;

import java.util.*;
import vio.tracker.*;

public class TrackerData {

	public short yaw, pitch, roll;
	public float yawAngle, pitchAngle, rollAngle;

	private String yawString, pitchString, rollString;

	private byte[] binaryTrackerData;
	private String asciiTrackerData;
	private byte[] yawWord, pitchWord, rollWord;
	private byte startByte, checksumByte;

	// constructor for a zero data object
	public TrackerData () {
		yawAngle   = 0.0f;
		pitchAngle = 0.0f;
		rollAngle  = 0.0f;
	}

	public TrackerData (String asciiTrackerData) throws TrackerDataException {

		this.asciiTrackerData = asciiTrackerData;
		
		try {
			StringTokenizer strt = new StringTokenizer(asciiTrackerData," ");
			if (strt.nextToken().equals("FF")) {
				yawString   = "0x"+strt.nextToken();
				pitchString = "0x"+strt.nextToken();
				rollString  = "0x"+strt.nextToken();
			}
			else { throw new TrackerDataException(); }
		}
		catch (NoSuchElementException e) { throw new TrackerDataException(); }

		try {
			yaw   = (short)Integer.decode(yawString).shortValue();
			pitch = (short)Integer.decode(pitchString).shortValue();
			roll  = (short)Integer.decode(rollString).shortValue();
		
			yawAngle   = (float)yaw   /(float)(16384.0/180.0);
			pitchAngle = (float)pitch /(float)(16384.0/180.0);
			rollAngle  = (float)roll  /(float)(16384.0/180.0);
		}
		catch (Exception e) { throw new TrackerDataException(); }
	}

	public TrackerData (byte[] binaryTrackerData) throws TrackerDataException {

		this.binaryTrackerData = binaryTrackerData;
		
		try {
			if (binaryTrackerData[0]==-1) {
				yaw   = glue(binaryTrackerData[1],binaryTrackerData[2]);
				pitch = glue(binaryTrackerData[3],binaryTrackerData[4]);
				roll  = glue(binaryTrackerData[5],binaryTrackerData[6]);

				yawAngle   = (float)yaw   / (float)(16384.0/180.0);
				pitchAngle = (float)pitch / (float)(16384.0/180.0);
				rollAngle  = (float)roll  / (float)(16384.0/180.0);
			}
			else { throw new TrackerDataException(); }

		}
		catch (Exception e) { throw new TrackerDataException(); }
	}

	public boolean equals (TrackerData td) {
		boolean eq=false;		
		if ( (this.yaw==td.yaw) && (this.pitch==td.pitch) && (this.roll==td.roll) )
			eq=true;
			
		return eq;
	}
	
	public void printData () {
		System.out.println(yawAngle+"\t"+pitchAngle+"\t"+rollAngle);
	}
	
	private short glue(byte a, byte b) {
		if (b>=0)
			return (short)(a*256|b);
		else
			return (short)((a*256+255)&b);
	}

}
