package vio.util;

import vio.tracker.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class j3dTrackerInputDevice implements InputDevice {

	private TrackerManager tracker;
	private TrackerData myTrackerData   = new TrackerData();
	private TrackerData lastTrackerData = new TrackerData();
	private TrackerData zeroTrackerData = new TrackerData();
	private String trackerPortName = "";

	private float yaw   = 0.0f;
	private float pitch = 0.0f;
	private float roll  = 0.0f;		

	private float pi=(float)Math.PI;
	
    	private Sensor sensors[] = new Sensor[1];
     	private SensorRead sensorRead = new SensorRead();
   	private int processingMode;

   	private Vector3f position = new Vector3f();

    	private Transform3D newTransform = new Transform3D();
	private Transform3D rotTransX = new Transform3D();
	private Transform3D rotTransY = new Transform3D();
	private Transform3D rotTransZ = new Transform3D();

	private float multiplicator=(float)(-2*Math.PI/360);
	
	private Thread myThread = null;
	
	public j3dTrackerInputDevice( TrackerManager tracker ) {
		
		//position.set(pi, pi, pi);
		position.set(0.0f, 0.0f, 1.0f);
		
		this.tracker=tracker;
		trackerPortName=tracker.getTrackerPortName();
		
		processingMode = InputDevice.DEMAND_DRIVEN;
		//processingMode = InputDevice.NON_BLOCKING;
		sensors[0] = new Sensor(this);

	}

	public boolean initialize() {
		return true;
	}

	public void close() {
		tracker.close();
		tracker=null;
	}

	public void pollAndProcessInput() {
	   	sensorRead.setTime( System.currentTimeMillis() );
	
		try {
			myTrackerData = tracker.getTrackerData();
			yaw = myTrackerData.yawAngle*multiplicator;
			pitch = myTrackerData.pitchAngle*multiplicator;
			//roll = myTrackerData.rollAngle*multiplicator;			
		}
		catch (TrackerManagerException tme) {
			tracker.close();
			tracker = null;
			boolean trackerAvailable = false;
			while (!trackerAvailable) {
				tracker = new TrackerManager(trackerPortName,false);
				trackerAvailable=tracker.init();
				
				if (!trackerAvailable)
					System.out.println("please check the tracker ...");
			}
		}

	        rotTransX.rotX(-pitch);
	        rotTransY.rotY(-yaw);
	        //rotTransZ.rotZ(-roll);
		
		newTransform.set(position);
        
	        newTransform.mul(rotTransX );
	        newTransform.mul(rotTransY);
	        //newTransform.mul(rotTransZ);
	
	        sensorRead.set( newTransform );
	        sensors[0].setNextSensorRead( sensorRead );
	}

	public void processStreamInput() {
	}

	public int getSensorCount() {
		return sensors.length;
	}

	public Sensor getSensor( int sensorIndex ) {
		return sensors[sensorIndex];
	}

	public int getProcessingMode() {
	        return processingMode;
	}
	
	public void setProcessingMode( int mode ) {
	         // A typical driver might implement only one of these modes, and
	         // throw an exception when there is an attempt to switch modes.
	         // However, this example allows one to use any processing mode.	
	         switch(mode) {
		            case InputDevice.DEMAND_DRIVEN:
		            case InputDevice.NON_BLOCKING:
		            case InputDevice.BLOCKING:
		                 processingMode = mode;
		            break;
		            default:
		               throw new IllegalArgumentException("Processing mode must " +
		                       "be one of DEMAND_DRIVEN, NON_BLOCKING, or BLOCKING");
	         }
    	}

	public void setNominalPositionAndOrientation() {
	        sensorRead.setTime( System.currentTimeMillis() );
	 
	        rotTransX.rotX(0.0f);
	        rotTransY.rotY(0.0f);
	        rotTransZ.rotZ(0.0f);
	 	 
	        newTransform.mul(rotTransX);  
	        newTransform.mul(rotTransY);
	        newTransform.mul(rotTransZ);
	 
	        sensorRead.set( newTransform );
	        sensors[0].setNextSensorRead( sensorRead );
	}

}
